@@ -444,28 +444,23 @@ log_msg_set_value(LogMessage *self, NVHandle handle, const gchar *value, gssize
       log_msg_set_flag(self, LF_STATE_OWN_PAYLOAD);
     }
 
-  do
+  /* we need a loop here as a single realloc may not be enough. Might help
+   * if we pass how much bytes we need though. */
+
+  while (!nv_table_add_value(self->payload, handle, name, name_len, value, value_len, &new_entry))
     {
-      if (!nv_table_add_value(self->payload, handle, name, name_len, value, value_len, &new_entry))
-        {
-          /* error allocating string in payload, reallocate */
-          if (!nv_table_realloc(self->payload, &self->payload))
-            {
-              /* can't grow the payload, it has reached the maximum size */
-              msg_info("Cannot store value for this log message, maximum size has been reached",
-                       evt_tag_str("name", name),
-                       evt_tag_printf("value", "%.32s%s", value, value_len > 32 ? "..." : ""),
-                       NULL);
-              break;
-            }
-          stats_counter_inc(count_payload_reallocs);
-        }
-      else
+      /* error allocating string in payload, reallocate */
+      if (!nv_table_realloc(self->payload, &self->payload))
         {
+          /* can't grow the payload, it has reached the maximum size */
+          msg_info("Cannot store value for this log message, maximum size has been reached",
+                   evt_tag_str("name", name),
+                   evt_tag_printf("value", "%.32s%s", value, value_len > 32 ? "..." : ""),
+                   NULL);
           break;
         }
+      stats_counter_inc(count_payload_reallocs);
     }
-  while (1);
 
   if (new_entry)
     log_msg_update_sdata(self, handle, name, name_len);
@@ -493,28 +488,20 @@ log_msg_set_value_indirect(LogMessage *self, NVHandle handle, NVHandle ref_handl
       log_msg_set_flag(self, LF_STATE_OWN_PAYLOAD);
     }
 
-  do
+  while (!nv_table_add_value_indirect(self->payload, handle, name, name_len, ref_handle, type, ofs, len, &new_entry))
     {
-      if (!nv_table_add_value_indirect(self->payload, handle, name, name_len, ref_handle, type, ofs, len, &new_entry))
-        {
-          /* error allocating string in payload, reallocate */
-          if (!nv_table_realloc(self->payload, &self->payload))
-            {
-              /* error growing the payload, skip without storing the value */
-              msg_info("Cannot store referenced value for this log message, maximum size has been reached",
-                       evt_tag_str("name", name),
-                       evt_tag_str("ref-name", log_msg_get_value_name(ref_handle, NULL)),
-                       NULL);
-              break;
-            }
-          stats_counter_inc(count_payload_reallocs);
-        }
-      else
+      /* error allocating string in payload, reallocate */
+      if (!nv_table_realloc(self->payload, &self->payload))
         {
+          /* error growing the payload, skip without storing the value */
+          msg_info("Cannot store referenced value for this log message, maximum size has been reached",
+                   evt_tag_str("name", name),
+                   evt_tag_str("ref-name", log_msg_get_value_name(ref_handle, NULL)),
+                   NULL);
           break;
         }
+      stats_counter_inc(count_payload_reallocs);
     }
-  while (1);
 
   if (new_entry)
     log_msg_update_sdata(self, handle, name, name_len);
