@@ -380,6 +380,9 @@ pdbtool_pdb_emit(LogMessage *msg, gboolean synthetic, gpointer user_data)
             printf("\nValues:\n");
 
           nv_table_foreach(msg->payload, logmsg_registry, pdbtool_match_values, ret);
+          g_string_truncate(output, 0);
+          log_msg_print_tags(msg, output);
+          printf("TAGS=%s\n", output->str);
           printf("\n");
         }
       else
@@ -428,8 +431,8 @@ pdbtool_match(int argc, char *argv[])
       log_template_compile(template, t, NULL);
       g_free(t);
 
-      output = g_string_sized_new(512);
     }
+  output = g_string_sized_new(512);
 
   if (filter_string)
     {
@@ -684,6 +687,13 @@ pdbtool_test(int argc, char *argv[])
 
   for (arg_pos = 1; arg_pos < argc; arg_pos++)
     {
+      if (access(argv[arg_pos], R_OK) == -1)
+	{
+	  fprintf(stderr, "%s: Unable to access the patterndb file\n", argv[arg_pos]);
+	  failed_to_validate = TRUE;
+	  continue;
+	}
+
       if (test_validate)
         {
           gchar cmd[1024];
@@ -694,12 +704,14 @@ pdbtool_test(int argc, char *argv[])
             {
               fprintf(stderr, "%s: Unable to detect patterndb version, please write the <patterndb> tag on a single line\n", argv[arg_pos]);
               failed_to_validate = TRUE;
+	      continue;
             }
           g_snprintf(cmd, sizeof(cmd), "xmllint --noout --nonet --schema %s/patterndb-%d.xsd %s", PATH_XSDDIR, version, argv[arg_pos]);
           if (system(cmd) != 0)
             {
               fprintf(stderr, "%s: xmllint returned an error, the executed command was: %s", argv[arg_pos], cmd);
               failed_to_validate = TRUE;
+	      continue;
             }
         }
 
@@ -762,7 +774,7 @@ pdbtool_test(int argc, char *argv[])
     return 1;
   if (failed_to_match)
     return 2;
-  if (failed_to_find_id)
+  if (failed_to_find_id && test_ruleid)
     {
       printf("Could not find the specified ID, or the defined rule doesn't have an example message.\n");
       return 3;
